import os
import json

from app.states.state import State


class NotSelectedState:
    def __init__(self, context):
        self.context = context

    def has_access_to_draft(self, record_id):
        return True

    def handle(self):
        self.clear_state()

        print("\nPlease create a draft or select an existing one.")
        while True:
            print("\nOptions:")
            print("  1. Create a new draft from JSON file")
            print("  2. Set draft by id")
            print("  3. Back to configuration")
            print("  9. Help")
            print("  0. Exit")

            choice = input("Select option: ").strip()
            if choice == '1':
                file_path = input("Enter path to JSON file: ").strip()
                if not os.path.isfile(file_path):
                    print("File does not exist.")
                    continue
                try:
                    with open(file_path, 'r') as f:
                        body = json.load(f)
                    print("File loaded successfully.")
                except Exception as e:
                    print(f"Failed to load JSON: \n     {e}")
            elif choice == '2':
                record_id = input("Enter record id: ").strip()
                if self.has_access_to_draft(record_id):
                    self.context.selected_id = record_id
                    print(f"Selected id set to: {record_id}")
                    self.context.last_state = State.NOT_SELECTED
                    self.context.state = State.SELECTED
                    break
                else:
                    print("Access denied or invalid id.")
            elif choice == '3':
                self.context.last_state = State.NOT_SELECTED
                self.context.state = State.CONFIGURATION
                print("Returning to configuration state.")
                break
            elif choice == '9':
                self.context.last_state = State.NOT_SELECTED
                self.context.state = State.HELP
                break
            elif choice == '0':
                exit(0)
            else:
                print("Invalid option. Please try again.")

    def clear_state(self):
        self.context.selected_id = None
        self.context.json_body = None
        self.context.selected_data = None



import configparser
import os

from .base_state import BaseState
from .state import State

CONFIG_FILE = 'config.cfg'
REQUIRED_KEYS = ['BASE_API_URL', 'API_TOKEN', 'MODEL']


class ConfigurationState(BaseState):
    def __init__(self, context):
        super().__init__(context)
        self.config_parser = configparser.ConfigParser()
        self.load_config()

    def load_config(self):
        if os.path.exists(CONFIG_FILE):
            self.config_parser.read(CONFIG_FILE)
            if 'DEFAULT' in self.config_parser:
                for key in REQUIRED_KEYS:
                    self.context.config[key] = self.config_parser['DEFAULT'].get(key)
        else:
            for key in REQUIRED_KEYS:
                self.context.config[key] = None

    def save_config(self):
        self.config_parser['DEFAULT'] = {k: v or '' for k, v in self.context.config.items()}
        with open(CONFIG_FILE, 'w') as configfile:
            self.config_parser.write(configfile)

    def is_config_complete(self):
        return all(self.context.config.get(k) for k in REQUIRED_KEYS)

    def show_config(self):
        print("\nCurrent configuration:")
        for key in REQUIRED_KEYS:
            value = self.context.config.get(key)
            print(f"  {key}: {value if value else '[NOT SET]'}")
        if self.is_config_complete():
            print("\n✅ Configuration is complete.")
        else:
            print("\n❌ Configuration is incomplete. Please set all values.")

    def handle(self):
        self.context.last_state = State.CONFIGURATION

        while True:
            self.show_config()
            print("\nOptions:")
            for idx, key in enumerate(REQUIRED_KEYS, 1):
                print(f"  {idx}. Set {key}")
            if self.is_config_complete():
                print(f"  {len(REQUIRED_KEYS) + 1}. Continue")
            print("  9. Help")
            print("  0. Exit")

            choice = input("Select option: ").strip()
            if choice == '0':
                exit(0)
            elif choice in map(str, range(1, len(REQUIRED_KEYS) + 1)):
                key = REQUIRED_KEYS[int(choice) - 1]
                value = input(f"Enter value for {key}: ").strip()
                self.context.config[key] = value
                self.save_config()
            elif self.is_config_complete() and choice == str(len(REQUIRED_KEYS) + 1):
                self.save_config()
                self.context.state = State.NOT_SELECTED
                print("Configuration complete.")
                break
            elif choice == '9':
                self.context.state = State.HELP
                break
            else:
                print("Invalid option.")



from app.help_messages import help_config_state
from app.states.base_state import BaseState


class HelperState(BaseState):
    print("\nHelp:")

    def handle(self):
        while True:
            print("\nOptions:")
            print("  1. Configuration help")
            print("  9. Back")
            print("  0. Exit")

            choice = input("Select option: ").strip()
            if choice == '1':
                help_config_state()
            elif choice == '9':
                self.context.state = self.context.last_state
                break
            elif choice == '0':
                exit(0)
            else:
                print("Invalid option. Please try again.")



#
# Copyright (C) 2025 Masaryk University
#
# DAR-CLI is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.


from setuptools import setup

setup()
